// -*- Mode: vala; indent-tabs-mode: nil; tab-width: 4 -*-
/***
  BEGIN LICENSE

  Copyright (C) 2012 Mario Guerriero <mefrio.g@gmail.com>
  This program is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License version 3, as
  published    by the Free Software Foundation.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranties of
  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
  PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program.  If not, see <http://www.gnu.org/licenses>

  END LICENSE
***/

using Gtk;
using Gst;

namespace Smiler {
    
    public class TakeWindow : Gtk.Window {
    
        // Main elements
        DrawingArea drawing_area;
        Element camerabin;
        Gst.Bus bus;
        
        string filename;
        
        public signal void mediasaved (string path);
        
        public TakeWindow () {

            title = _("Take a new photo");
            set_default_size (450, 350);
            window_position = WindowPosition.CENTER;
            
            var mediabin = new MediaBin ();
            this.drawing_area = mediabin.preview_area;

            // Main elements to show the webcam stream
            this.camerabin = ElementFactory.make ("camerabin", "video");

            bus = this.camerabin.get_bus ();
            bus.set_sync_handler (on_bus_callback);
            bus.add_signal_watch();
            
            // UI
            var box = new Box (Orientation.VERTICAL, 0);
            
            var tb = new Toolbar ();
            tb.get_style_context ().add_class (Gtk.STYLE_CLASS_PRIMARY_TOOLBAR);
            
            var b = new Button.from_stock (Stock.MEDIA_RECORD);//.with_label (_("Take"));
            b.clicked.connect (() => {
                take_photo ();
            });
            var bbox = new ButtonBox (Orientation.HORIZONTAL);
            bbox.add (b);
            
            var s = new ToolItem ();
            s.set_expand (true);
            
            tb.add (s);
            
            var i = new ToolItem ();
            i.add (bbox);
            
            tb.add (i);
            
            s = new ToolItem ();
            s.set_expand (true);
            
            tb.add (s);
            
            box.pack_start (tb, false);
            box.pack_start (mediabin);
            
            this.add (box);

        }

        Gst.BusSyncReply on_bus_callback (Gst.Bus bus, Gst.Message message) {
            if (message.get_structure () != null &&
                message.get_structure().has_name("prepare-xwindow-id") &&
                this.drawing_area.get_window () != null)
            {
                var xoverlay = message.src as XOverlay;
                xoverlay.set_xwindow_id (Gdk.X11Window.get_xid (this.drawing_area.get_window ()));
                return Gst.BusSyncReply.DROP;
            }
            else return Gst.BusSyncReply.PASS;
            
        }

        public void switch_mode (int mode) {
            this.camerabin.set_property ("mode", mode);
        }

        public void play () {
            this.camerabin.set_state (State.PLAYING);
        }

        public void pause () {
            this.camerabin.set_state (State.PAUSED);
        }

        public void stop () {
            this.camerabin.abort_state ();
            this.camerabin.set_state (State.NULL);
            this.camerabin.unref ();
            Gst.deinit ();
        }

        public void take_photo () {
            debug ("Taking a photo...");

            this.filename = get_new_media_filename ();

            this.camerabin.set_property ("mode", 0);

            camerabin.set_property ("filename", filename);
            bus.message.connect(on_media_saved);
            GLib.Signal.emit_by_name (camerabin, "capture-start");

        }

        public void on_media_saved (Gst.Bus bus, Gst.Message msg) {
            string msgtype = msg.get_structure().get_name();
            if(msgtype == "image-captured") {
                mediasaved(filename);
                bus.message.disconnect(on_media_saved);
            }
        }
        
        public string get_media_dir () {
            UserDirectory user_dir;

            user_dir = UserDirectory.PICTURES;

            string dir = GLib.Environment.get_user_special_dir (user_dir);
            
            return dir;
        }
        
        public string get_new_media_filename (string? ext = null) {
            // Get date and time
            var datetime = new GLib.DateTime.now_local ();
            string time = datetime.format ("%F %H:%M:%S");

            int n = 0;
            string filename = "";
            do {
                filename = "smiler-" + n.to_string () + ".jpg";//time + (n > 0 ? " - " + n.to_string () : "" + ".jpg");
                n++;
            } while (GLib.FileUtils.test (build_media_filename (filename, ext), FileTest.EXISTS));

            return build_media_filename (filename, ext);
        }
        
        public string build_media_filename (string filename, string? ext = null) {
            string new_filename = "";
            if (ext == null) {
                new_filename = filename + ".jpg";
            } else {
                new_filename = filename + "." + ext;
            }

            return GLib.Path.build_filename ("/", get_media_dir (), filename);
        }
    
    }
    
    public class MediaBin : EventBox {

        public DrawingArea preview_area {get; private set;}

        private Box wrapper_box;

        // Aspect ratio hr:vr. Default: 4:3
        private int vr = 3;
        private int hr = 4;

        public MediaBin () {
            preview_area = new DrawingArea ();

            wrapper_box = new Box (Orientation.HORIZONTAL, 0);
            wrapper_box.pack_start (preview_area, true, true, 0);

            add (wrapper_box);
            
            const string PREVIEW_STYLESHEET = """
                .preview-bg {
                    background-color: #000;
                }
            """;
            
            var style = new Gtk.CssProvider ();
            try {
                style.load_from_data (PREVIEW_STYLESHEET, -1);
            } catch (Error e) {
                warning (e.message);
            }

            // Dark background
            this.get_style_context ().add_class ("preview-bg");
            this.get_style_context ().add_provider (style, STYLE_PROVIDER_PRIORITY_THEME);

            this.size_allocate.connect (on_size_allocate);
        }

        public void set_aspect_ratio (int horizontal, int vertical) {
            if (horizontal <= 0 || vertical <= 0)
                return;

            hr = horizontal;
            vr = vertical;
        }

        private void on_size_allocate (Allocation alloc) {
            Timeout.add (20, () => {
                int preview_width = (this.get_allocated_height() * hr) / vr;
                int blank_area_width = this.get_allocated_width() - preview_width;
                int margin = blank_area_width / 2;
/*
                if (blank_area_width < 0)
                    return false;

                if (preview_area.get_allocated_width () - 2 * margin < 0)
                    return false;
*/
                preview_area.margin_left = margin;
                preview_area.margin_right = margin;

                return preview_width / this.get_allocated_height() != 4 / 3;
            });
        }

    }
    
}
