// -*- Mode: vala; indent-tabs-mode: nil; tab-width: 4 -*-
/***
  BEGIN LICENSE

  Copyright (C) 2012 Mario Guerriero <mefrio.g@gmail.com>
  This program is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License version 3, as
  published    by the Free Software Foundation.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranties of
  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
  PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program.  If not, see <http://www.gnu.org/licenses>

  END LICENSE
***/

using Gtk;

using Smiler.Utils;

namespace Smiler {

    public class SmilerWindow : Gtk.Window {
        
        private Smiler app;

        // UI and Actions things
        const string ui_string = MENUBAR;

        public Gtk.ActionGroup main_actions;
        public Gtk.UIManager ui;
        
        private EffectManager manager;
        private Effect selected_effect;
        
        private string origin = null;
        private string dest = null;
        
        private Box mbox;
        private Box wbox;
        
        private Image image;
        private ListStore store;
        private IconView effect_view;
        
        public SmilerWindow (Smiler app) {
            
            this.app = app;
            
            title = "Smiler";
            icon_name = "face-smile";
            set_default_size (400, 370);
            window_position = WindowPosition.CENTER;
            
            add_events (Gdk.EventMask.BUTTON_PRESS_MASK | Gdk.EventMask.POINTER_MOTION_MASK);

            destroy.connect (action_close);
            
            // Init UI and Actions
            main_actions = new Gtk.ActionGroup ("MainActionGroup");
            main_actions.set_translation_domain ("smiler");
            main_actions.add_actions (main_entries, this);
            
            ui = new Gtk.UIManager ();

            try {
                ui.add_ui_from_string (ui_string, -1);
            }
            catch(Error e) {
                error ("Couldn't load the UI: %s", e.message);
            }

            Gtk.AccelGroup accel_group = ui.get_accel_group();
            add_accel_group (accel_group);

            ui.insert_action_group (main_actions, 0);
            ui.ensure_update ();
            
            // Manager
            manager = new EffectManager ();
            manager.effect_loaded.connect ((e) => {
                GLib.Timeout.add (10, () => {
                    update_image ();
                    return false;
                });
            });
            
            // Setup window's ui
            setup_ui ();
            
            // Setup effect
            foreach (var e in manager.effects)
                add_effect (e);
             
        }
        
        private void setup_ui () {
            // Main box
            var box = new Box (Orientation.VERTICAL, 0);
            
            // MenuBar
            var menubar = ui.get_widget ("ui/MenuBar");
            
            // Welcome
            wbox = new Box (Orientation.VERTICAL, 0);
            
            var welcome = new Welcome (_("Welcome to Smiler!"), _("Smiler is a simple app that allow you to use fun\neffects on your photos.\nTo get started you can\ntake a new photo using a webcam or,\neasily, you can open an existing image."));
            welcome.halign = Align.CENTER;
            welcome.valign = Align.CENTER;
            
            welcome.append ("gtk-open", _("Open an image"), "");
            welcome.append ("camera-web", _("Take from webcam"), "");
            
            // Check if there is webcam
            try {
                string webcam = null;
                GLib.Process.spawn_command_line_sync ("python " + WEBCAMSCRIPT, out webcam);
                stdout.printf ("%s\n", webcam);
                bool is_webcam = true;
                if (webcam == null || webcam == "")
                    is_webcam = false;
                welcome.set_item_sensitivity (1, is_webcam);
            } catch (SpawnError e) {
                stderr.printf ("%s\n", e.message);
            }
            
            welcome.activated.connect ((i) => {
                if (i == 0) { // Open
                    var filter = new FileFilter ();
                    filter.add_mime_type ("image/*");
                    var file_chooser = new FileChooserDialog ("Choose a photo", this,
                                              FileChooserAction.OPEN,
                                              Stock.CANCEL, ResponseType.CANCEL,
                                              Stock.OPEN, ResponseType.ACCEPT);
                    file_chooser.filter = filter;
                    if (file_chooser.run () == ResponseType.ACCEPT) {
                        open (file_chooser.get_filename ());
                        box.remove (wbox);
                        box.pack_start (mbox, true, true);   
                        show_all ();
                    }
                    file_chooser.destroy ();
                }
                else if (i == 1) {
                    var tw = new TakeWindow ();
                    tw.show_all ();
                    tw.play ();
                    tw.mediasaved.connect ((p) => {
                        open (p);
                        tw.destroy ();
                        box.remove (wbox);
                        box.pack_start (mbox, true, true);   
                        show_all ();
                    });
                }
                
            });
            
            wbox.pack_start (welcome, true, true);
            
            // Toolbar 
            var tb = new Toolbar ();
            tb.get_style_context ().add_class ("primary-toolbar");
            
            var saveas = new ToolButton.from_stock (Gtk.Stock.SAVE);
            saveas.tooltip_text = _("Save this image file in a different place");
            saveas.clicked.connect (() => {
                save_as ();
            });
            
            var revert = new ToolButton.from_stock (Gtk.Stock.REVERT_TO_SAVED);
            revert.tooltip_text = _("Revert image state to the original one");
            revert.clicked.connect (() => {
                restore_image ();
            });
            
            tb.add (saveas);
            tb.add (revert);
            
            // Central box
            image = new Image ();
            image.margin_bottom = 30;
            image.margin_top = 15;
            
            var s = new Separator (Orientation.HORIZONTAL);
            
            store = new Gtk.ListStore (3, typeof (string), typeof (Gdk.Pixbuf), typeof (Effect));
    
            effect_view = new IconView ();
            effect_view.set_columns (999); // A workaround
            effect_view.set_selection_mode (Gtk.SelectionMode.SINGLE);
            effect_view.set_tooltip_column (0);
            effect_view.set_pixbuf_column (1);
            effect_view.set_model (store);
            effect_view.selection_changed.connect (() => {
                var item = effect_view.get_selected_items ();
                if (item != null) {
                    TreeIter iter;

                    store.get_iter_from_string(out iter, item.nth_data (0).to_string());

                    store.get (iter, 2, out selected_effect);

                }
            });
            effect_view.item_activated.connect (() => {
                manager.apply_effect (selected_effect, origin, dest);
                // Show animation
                Allocation al;
                int x, y, h, w;
                image.get_window ().get_origin (out x, out y);
                //image.get_size_request (out w, out h);
                image.get_allocation (out al);
                h = al.height;
                w = al.width;
                stdout.printf ("%d\n", h);
                new PoofWindow (x + (h/2), y + (w/2));
            });
            
            var scroll = new ScrolledWindow (null, null);
            scroll.vscrollbar_policy = PolicyType.NEVER;
            scroll.add (effect_view);
            
            // Append everything to the window
            mbox = new Box (Orientation.VERTICAL, 0);
            
            mbox.pack_start (menubar, false, false, 0);
            mbox.pack_start (tb, false, false, 0);
            mbox.pack_start (image, true, true, 0);
            mbox.pack_start (s, false, false, 0);
            mbox.pack_start (scroll, false, false, 5);
            
            //box.pack_start (mbox, true, true); // Not for now
            box.pack_start (wbox, true, true);
            
            // Set some widgets insensitive before open an image
            effect_view.set_sensitive (false);
            main_actions.get_action ("UOne").set_sensitive (false);
            
            this.add (box);
            show_all ();

        }
        
        void add_effect (Effect eff) {
            TreeIter iter;
            this.store.append (out iter);
            
            Gdk.Pixbuf? pix = null;
            
            try {
                pix = new Gdk.Pixbuf.from_file_at_size (eff.path, 64, 64);
            } catch (Error e) {
                stderr.printf ("%s\n", e.message);
                return;
            }
    
            this.store.set (iter, 0, eff.name, 1, pix, 2, eff);
            
            effect_view.show_all ();
        }
        
        void update_image () {
            try {
                var pix = new Gdk.Pixbuf.from_file (dest);
                image.pixbuf = pix.scale_simple (600, 500, 0);
            } catch (Error e) {
                stderr.printf ("%s\n", e.message);
            }
        }
        
        void load_image (string path) {
            try {
                var pix = new Gdk.Pixbuf.from_file (path);
                image.pixbuf = pix.scale_simple (600, 500, 0);
            } catch (Error e) {
                stderr.printf ("%s\n", e.message);
            }
        }
        
        void restore_image () {
            var fo = File.new_for_path (origin);
            var fd = File.new_for_path (dest);
            fo.copy (fd, FileCopyFlags.OVERWRITE);
            update_image ();
        }
        
        void save_as () {
            var filter = new FileFilter ();
            filter.add_mime_type ("image/*");
            var file_chooser = new FileChooserDialog ("Choose a photo", this,
                                      FileChooserAction.SAVE,
                                      Stock.CANCEL, ResponseType.CANCEL,
                                      Stock.OPEN, ResponseType.ACCEPT);
            file_chooser.filter = filter;
            file_chooser.set_filename (dest);
            if (file_chooser.run () == ResponseType.ACCEPT) {
                string new_dest = file_chooser.get_filename ();
                var o = File.new_for_path (dest);
                var d = File.new_for_path (new_dest);
                o.copy (d, FileCopyFlags.OVERWRITE);
                dest = new_dest;
            }
            file_chooser.destroy ();
        }
        
        void open (string path) {
            try {
                dest = path;
                //Create a backup copy of the file
                var fd = File.new_for_path (dest);
                    
                origin = dest + "~";
                var fo = File.new_for_path (origin);
                fd.copy (fo, FileCopyFlags.OVERWRITE);
                    
                var pix = new Gdk.Pixbuf.from_file (origin);
                image.pixbuf = pix.scale_simple (600, 500, 0);
                effect_view.set_sensitive (true);
                main_actions.get_action ("UOne").set_sensitive (true);
            } catch (Error e) {
                stderr.printf ("%s\n", e.message);
            }
        }
        
        // Actions functions
        void action_close () {
            destroy ();
            // Delete normal files
            if (origin != null) {
                var f = File.new_for_path (origin);
                f.delete ();
            }
            Gtk.main_quit ();
        }
        
        void action_open () {
            var filter = new FileFilter ();
            filter.add_mime_type ("image/*");
            var file_chooser = new FileChooserDialog ("Choose a photo", this,
                                      FileChooserAction.OPEN,
                                      Stock.CANCEL, ResponseType.CANCEL,
                                      Stock.OPEN, ResponseType.ACCEPT);
            file_chooser.filter = filter;
            if (file_chooser.run () == ResponseType.ACCEPT) {
                open (file_chooser.get_filename ());
            }
            file_chooser.destroy ();
        }
        
        void action_take () {
            var w = new TakeWindow ();
            w.show_all ();
            w.play ();
            w.mediasaved.connect ((p) => {
                open (p);
                w.destroy ();
            });
        }
        
        void action_social () {
            try {
    	        GLib.Process.spawn_command_line_async ("python " + IMGURSCRIPT + " " + dest);   
    	    } catch (SpawnError e) {
    	        stderr.printf ("%s\n", e.message);
            }
        }
        
        void action_uone () {
            try {
    	        var f = File.new_for_path (dest);
    	        string exec = "python /opt/extras.ubuntu.com/smiler/script/u1file.py put " + dest.replace (" ", "\\ ").replace (":", "\\:") + " Ubuntu\\ One/" + f.get_basename ();
    	        stdout.printf ("%s\n", exec);
    	        GLib.Process.spawn_command_line_async (exec);   
    	    } catch (SpawnError e) {
    	        stderr.printf ("%s\n", e.message);
            }
        }
        
        void show_about () {
            var about = new AboutDialog ();
            about.response.connect (() => { about.destroy (); });
            about.program_name = APP_TITLE;
            about.version = Constants.VERSION;
            about.comments = COMMENTS;
            about.copyright = COPYRIGHTS;
            about.license = LICENSE;
            about.wrap_license = true;
            about.website = LAUNCHPAD_URL;
            about.authors = AUTHORS;
            about.logo_icon_name = ICON_ABOUT_LOGO;
            about.translator_credits = TRANSLATORS;
            about.run ();
        }
        
        void show_report () {
            try { Gtk.show_uri (null, BUG_URL, -1); }
            catch (Error e) { critical("Can't open the link"); }
        }

        void show_translate() {
            try { Gtk.show_uri (null, TRANSLATE_URL, -1); }
            catch (Error e) { critical("Can't open the link"); }
        }

        void show_app_help() {
            try { Gtk.show_uri (null, HELP_URL, -1); }
            catch (Error e) { critical("Can't open the link"); }
        }
        
        static const Gtk.ActionEntry[] main_entries = {
  /* name, stock id, label */  { "File", null, N_("_File") },
  /* name, stock id, label */  { "Help", null, N_("_Help") },
  /* name, stock id, label */  { "Share", null, N_("_Share") },
// File actions
  /* name, stock id */         { "Open", Stock.OPEN,
  /* label, accelerator */       N_("_Open"), "<control>O",
  /* tooltip */                  N_("Open an image"),
                                 action_open },
  /* name, stock id */         { "TakeNew", null,
  /* label, accelerator */       N_("Take a new photo"), null,
  /* tooltip */                  N_("Take a new photo from webcam"),
                                 action_take },
  /* name, stock id */         { "Close", Stock.CLOSE,
  /* label, accelerator */       N_("_Close"), "<control>Q",
  /* tooltip */                  N_("Close"),
                                 action_close },
// Share actions
  /* name, stock id */         { "Social", null,
  /* label, accelerator */       N_("Share on social networks"), null,
  /* tooltip */                  N_("Share on social networks"),
                                 action_social },
  /* name, stock id */         { "UOne", null,
  /* label, accelerator */       N_("Send to your Ubuntu One folder"), null,
  /* tooltip */                  N_("Send to your Ubuntu One folder"),
                                 action_uone },
// Help actions
  /* name, stock id */         { "About", Stock.ABOUT,
  /* label, accelerator */       N_("_About"), null,
  /* tooltip */                  N_("Display credits"),
                                 show_about },
  /* name, stock id */         { "ReportProblem", "",
  /* label, accelerator */       N_("Report a Problem..."), null,
  /* tooltip */                  N_("File a bug on Launchpad"),
                                 show_report },
  /* name, stock id */         { "GetHelp", "",
  /* label, accelerator */       N_("Get Help Online..."), null,
  /* tooltip */                  "",
                                 show_app_help },
  /* name, stock id */         { "Translate", "",
  /* label, accelerator */       N_("Translate This Application..."), null,
  /* tooltip */                  "",
                                 show_translate }
        };
    
    }
    
}
